@@ -5,24 +5,17 @@
   ## This function should not be called by the user.
   ## It displays a message when the package is being loaded.
   packageStartupMessage(## display message
-                        "\n IsoriX version ", utils::packageDescription("IsoriX")$Version," is now loaded!",
+                        "\n IsoriX version ", utils::packageDescription("IsoriX")$Version," is now loaded",
                         "\n",
                         "\n Type:",
+                        "\n   * `?IsoriX` for a very short description",
+                        "\n   * `browseURL('https://bookdown.org/content/782/')` for a longer (online) documentation",
+                        "\n   * `help(package = 'IsoriX', help_type = 'html')` for a list of the package objects and help files",
+                        "\n   * `citation('IsoriX')` for how to cite IsoriX (i.e. the papers you should read)",
+                        "\n   * `news(package = 'IsoriX')` for info on changed between versions of IsoriX",
                         "\n",
-                        "\n    * ?IsoriX",
-                        "\n      for a very short description",
-                        "\n",
-                        "\n    * help(package = 'IsoriX', help_type = 'html')",
-                        "\n      for a list of the package objects and their help files",
-                        "\n",
-                        "\n    * browseURL('https://bookdown.org/content/782/')",
-                        "\n      for online tutorials & documentation",
-                        "\n",
-                        "\n    * citation('IsoriX')",
-                        "\n      for information on how to cite IsoriX (i.e. the papers you should read)",
-                        "\n",
-                        "\n    * news(package = 'IsoriX')",
-                        "\n      for info on what has changed between the different versions of IsoriX",
+                        "\n Please join the mailing list 'https://groups.google.com/g/IsoriX'",
+                        "\n for help, news and discussions about IsoriX",
                         "\n"
                         )
   }
@@ -167,7 +160,7 @@
 .summarize_values <- function(var, nb_quantiles = 1e4) {
   ## This function should not be called by the user.
   ## It extracts and summarizes the raster values using quantiles if needed.
-  if (!class(var) %in% c("RasterLayer", "RasterStack", "RasterBrick")) {
+  if (!inherits(var, c("RasterLayer", "RasterStack", "RasterBrick"))) {
     return(var)
   } else if (raster::inMemory(var)) {
     return(as.numeric(raster::values(var)))
@@ -177,10 +170,10 @@
     print("extracting values from stored rasters...")
   }
   
-  if (class(var) %in% c("RasterLayer")) {
+  if (inherits(var, c("RasterLayer"))) {
     var <- raster::quantile(var, seq(0, 1, length = nb_quantiles))
     return(var)
-  } else if (class(var) %in% c("RasterStack", "RasterBrick")) {
+  } else if (inherits(var, c("RasterStack", "RasterBrick"))) {
     max_var <- max(raster::maxValue(var))
     min_var <- min(raster::minValue(var))
     var <- unique(c(min_var,
@@ -190,3 +183,78 @@
   }
   stop("'var' has an unknown class")
 }
+
+
+.crop_withmargin <- function(raster, xmin, xmax, ymin, ymax, margin_pct = 5) {
+  ## This function should not be called by the user.
+  ## It crops a raster using a safety margin
+  margin_long <- (xmax - xmin) * margin_pct/100
+  margin_lat  <- (ymax - ymin) * margin_pct/100
+  
+  raster::crop(raster, raster::extent(xmin - margin_long,
+                                      xmax + margin_long,
+                                      ymin - margin_lat,
+                                      ymax + margin_lat))
+
+}
+
+.invert_reg <- function(intercept, slope, SE_I, SE_S, phi, N, sign_mean_Y) {
+  ## This function should not be called by the user.
+  ## It turns a regression x ~ y to a regression y ~ x
+  Nminus1 <- N - 1L
+  Nminus2 <- N - 2L
+  Nfac <- Nminus1/N
+  
+  MSExony <- phi
+  VarSxony <- SE_S^2
+
+  Vary <- MSExony/(Nminus1*VarSxony)
+  Covxy <- Vary*slope
+  Varx <- (MSExony*(slope^2 + Nminus2*VarSxony))/(Nminus1*VarSxony)
+  o_slope <- Covxy/Varx
+  
+  resid_MSE <-  (Vary - Covxy^2/Varx)*Nminus1/Nminus2
+  o_SE_S <- sqrt(resid_MSE/(Nminus1*Varx))
+  
+  Ey2 <- (SE_I/SE_S)^2 
+  Ey <- sign_mean_Y * sqrt(Ey2 - Vary*Nfac)
+  Ex <- intercept + slope*Ey
+  Ex2 <- Varx*Nfac + Ex^2
+  o_SE_I <- sqrt(resid_MSE*Ex2/(Nminus1*Varx))
+  vcov12 <- -resid_MSE*Ex/(Nminus1*Varx)
+  o_vcov <- matrix(c(o_SE_I^2, vcov12, vcov12, o_SE_S^2), ncol = 2)
+  
+  list(intercept = Ey - o_slope*Ex, 
+       slope = o_slope, 
+       SE_I = o_SE_I, 
+       SE_S = o_SE_S, 
+       phi = resid_MSE,
+       vcov = o_vcov)
+}
+# Example:
+# set.seed(123)
+# xy <- data.frame(x = x <- rnorm(20), y = rnorm(20, mean = 10) + 0.7*x)
+# input <- lm(x ~ y, data = xy)
+# output <- lm(y ~ x, data = xy)
+# 
+# foo <- .invert_reg(intercept = coef(input)[1],
+#                   slope = coef(input)[2],
+#                   SE_I = sqrt(vcov(input)[1, 1]),
+#                   SE_S = sqrt(vcov(input)[2, 2]),
+#                   phi = summary(input)$sigma^2,
+#                   sign_mean_Y = sign(mean(xy$y)),
+#                   N = 20)
+# 
+# d_output <- data.frame(intercept =  coef(output)[1],
+#                       slope = coef(output)[2],
+#                       SE_I = sqrt(vcov(output)[1, 1]),
+#                       SE_S = sqrt(vcov(output)[2, 2]),
+#                       phi = summary(output)$sigma^2)
+# 
+# d_foo <- data.frame(intercept =  foo$intercept,
+#                    slope = foo$slope,
+#                    SE_I = foo$SE_I,
+#                    SE_S = foo$SE_S,
+#                    phi = foo$phi)
+# 
+# rbind(d_output, d_foo)
